//For educational purposes only! 
//Do not rely on project to park car!
//Ultrasonic distance sensor and 4 wires required

//LCD display and its pins:
#include <LiquidCrystal.h>
LiquidCrystal lcd(29, 28, 27, 26, 25, 24); //RS, E, D4, D5, D6, D7

//Pins used for various components:
byte greenLight = 8;
byte yellowLight = 9;
byte redLight = 10;
byte backLight = 30;
byte buzzer = 31;
byte knob3 = 21;

byte triggerPin = 2;
byte echoPin = 3;

byte numberOfBlocks;
long timeOfFlight;
int distanceInCM;

// The setup function runs once on startup
void setup(){
  pinMode(greenLight, OUTPUT);
  pinMode(yellowLight, OUTPUT);
  pinMode(redLight, OUTPUT);
  pinMode(backLight, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(knob3, INPUT);

  lcd.begin(16, 2);//Initialize LCD screen (2 rows of 16 characters)
}

//The loop function repeats over and over. Every time it reaches the end it "loops back" to the beginning.
void loop(){
  
  //Trigger pulse to send out the sound wave
  digitalWrite(triggerPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(triggerPin, LOW); 

  timeOfFlight = pulseIn(echoPin, HIGH, 15000); //Measure the timeOfFlight from start of wave to return of the echo (in microseconds).
  if (timeOfFlight == 0){timeOfFlight = 999999;}

  distanceInCM = timeOfFlight * 0.0343 / 2; //Speed of sound 343 meters per second. PulseIn function measures timeOfFlight in microseconds (1 millionth of a second), so sound travels 0.0343 centimeters per microsecond. Because the sound pulse travels the distanceInCM between the sensor and object twice we need to divide the timeOfFlight by two.

  //UNCOMMENT THE NEXT LINE TO SIMULATE A DISTANCE MEASUREMENT AT YOUR DESK BY ROTATING KNOB 3
  //distanceInCM = (analogRead(knob3)/3); 

  
  //BEHAVIOR OF BACKLIGHT:
  if (distanceInCM > 300){digitalWrite(backLight, 0);}
  else {digitalWrite(backLight, 1);}

  //BEHAVIOR OF LEDS:
  if (distanceInCM > 100){digitalWrite(greenLight, LOW); digitalWrite(yellowLight, LOW); digitalWrite(redLight, LOW);} //No leds on
  else if ((distanceInCM > 70) && (distanceInCM <= 100)){digitalWrite(greenLight, HIGH); digitalWrite(yellowLight, LOW); digitalWrite(redLight, LOW);} //Green
  else if ((distanceInCM > 40) && (distanceInCM <= 70)){digitalWrite(greenLight, LOW); digitalWrite(yellowLight, HIGH); digitalWrite(redLight, LOW);} //Orange
  else if (distanceInCM < 40){digitalWrite(greenLight, LOW); digitalWrite(yellowLight, LOW); digitalWrite(redLight, HIGH);} //Red

  //BEHAVIOR OF BUZZER:
  if (distanceInCM < 40){tone(buzzer, 880);}
  else {noTone(buzzer);}

  //BEHAVIOR OF LCD PROGRESS BAR:
  if (distanceInCM > 100){
    numberOfBlocks = (distanceInCM - 90) / 10; if (numberOfBlocks > 16){numberOfBlocks = 16;}
    lcd.setCursor(0,1); for (byte q = 0; q < numberOfBlocks; q++){lcd.print((char)255);} for (byte p = 0; p < (16-numberOfBlocks); p++){lcd.print(" ");}  
    lcd.setCursor(0,0); for (byte q = 0; q < numberOfBlocks; q++){lcd.print((char)255);} for (byte p = 0; p < (16-numberOfBlocks); p++){lcd.print(" ");}}  
  else {lcd.clear();}

  //LCD distanceInCM MEASUREMENT DISPLAY
  lcd.setCursor(0,0); lcd.print(distanceInCM); lcd.print("cm  ");

  delay(50);
  
}
